@@ -14,7 +14,8 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
-from scipy.sparse.base import spmatrix
+from scipy.sparse import spmatrix
+
 from cvxpy.expressions import expression as exp
 
 BIN_OPS = ["__div__", "__mul__", "__add__", "__sub__",
