@@ -16,15 +16,18 @@ limitations under the License.
 THIS FILE IS DEPRECATED AND MAY BE REMOVED WITHOUT WARNING!
 DO NOT CALL THESE FUNCTIONS IN YOUR CODE!
 """
-import cvxpy.lin_ops.lin_op as lo
-from cvxpy.lin_ops.lin_constraints import LinEqConstr, LinLeqConstr
-import cvxpy.utilities as u
+from typing import Tuple
+
 import numpy as np
 
+import cvxpy.lin_ops.lin_op as lo
+import cvxpy.utilities as u
+from cvxpy.lin_ops.lin_constraints import LinEqConstr, LinLeqConstr
+
 # Utility functions for dealing with LinOp.
 
 
-class Counter(object):
+class Counter:
     """A counter for ids.
 
     Attributes
@@ -33,14 +36,14 @@ class Counter(object):
         The current count.
     """
 
-    def __init__(self):
-        self.count = 0
+    def __init__(self) -> None:
+        self.count = 1
 
 
 ID_COUNTER = Counter()
 
 
-def get_id():
+def get_id() -> int:
     """Returns a new id and updates the id counter.
 
     Returns
@@ -53,7 +56,7 @@ def get_id():
     return new_id
 
 
-def create_var(shape, var_id=None):
+def create_var(shape: Tuple[int, ...], var_id=None):
     """Creates a new internal variable.
 
     Parameters
@@ -73,13 +76,11 @@ def create_var(shape, var_id=None):
     return lo.LinOp(lo.VARIABLE, shape, [], var_id)
 
 
-def create_param(value, shape):
+def create_param(shape: Tuple[int, ...], param_id=None):
     """Wraps a parameter.
 
     Parameters
     ----------
-    value : CVXPY Expression
-        A function of parameters.
     shape : tuple
         The (rows, cols) dimensions of the operator.
 
@@ -88,10 +89,12 @@ def create_param(value, shape):
     LinOP
         A LinOp wrapping the parameter.
     """
-    return lo.LinOp(lo.PARAM, shape, [], value)
+    if param_id is None:
+        param_id = get_id()
+    return lo.LinOp(lo.PARAM, shape, [], param_id)
 
 
-def create_const(value, shape, sparse=False):
+def create_const(value, shape: Tuple[int, ...], sparse: bool = False):
     """Wraps a constant.
 
     Parameters
@@ -121,7 +124,7 @@ def create_const(value, shape, sparse=False):
     return lo.LinOp(op_type, shape, [], value)
 
 
-def is_scalar(operator):
+def is_scalar(operator) -> bool:
     """Returns whether a LinOp is a scalar.
 
     Parameters
@@ -136,7 +139,7 @@ def is_scalar(operator):
     return len(operator.shape) == 0 or np.prod(operator.shape, dtype=int) == 1
 
 
-def is_const(operator):
+def is_const(operator) -> bool:
     """Returns whether a LinOp is constant.
 
     Parameters
@@ -148,7 +151,10 @@ def is_const(operator):
     -------
         True if the LinOp is a constant, False otherwise.
     """
-    return operator.type in [lo.SCALAR_CONST, lo.SPARSE_CONST, lo.DENSE_CONST]
+    return operator.type in [lo.SCALAR_CONST,
+                             lo.SPARSE_CONST,
+                             lo.DENSE_CONST,
+                             lo.PARAM]
 
 
 def sum_expr(operators):
@@ -229,7 +235,7 @@ def promote_lin_ops_for_mul(lh_op, rh_op):
     return lh_op, rh_op, shape
 
 
-def mul_expr(lh_op, rh_op, shape):
+def mul_expr(lh_op, rh_op, shape: Tuple[int, ...]):
     """Multiply two linear operators, with the constant on the left.
 
     Parameters
@@ -247,7 +253,7 @@ def mul_expr(lh_op, rh_op, shape):
     return lo.LinOp(lo.MUL, shape, [rh_op], lh_op)
 
 
-def rmul_expr(lh_op, rh_op, shape):
+def rmul_expr(lh_op, rh_op, shape: Tuple[int, ...]):
     """Multiply two linear operators, with the constant on the right.
 
     Parameters
@@ -282,11 +288,12 @@ def multiply(lh_op, rh_op):
     LinOp
         A linear operator representing the product.
     """
-    return lo.LinOp(lo.MUL_ELEM, lh_op.shape, [rh_op], lh_op)
+    shape = max(lh_op.shape, rh_op.shape)
+    return lo.LinOp(lo.MUL_ELEM, shape, [rh_op], lh_op)
 
 
-def kron(lh_op, rh_op, shape):
-    """Kronecker product of two matrices.
+def kron_r(lh_op, rh_op, shape: Tuple[int, ...]):
+    """Kronecker product of two matrices, where the right operand is a Variable
 
     Parameters
     ----------
@@ -300,7 +307,25 @@ def kron(lh_op, rh_op, shape):
     LinOp
         A linear operator representing the Kronecker product.
     """
-    return lo.LinOp(lo.KRON, shape, [rh_op], lh_op)
+    return lo.LinOp(lo.KRON_R, shape, [rh_op], lh_op)
+
+
+def kron_l(lh_op, rh_op, shape: Tuple[int, ...]):
+    """Kronecker product of two matrices, where the left operand is a Variable
+
+    Parameters
+    ----------
+    lh_op : LinOp
+        The left-hand operator in the product.
+    rh_op : LinOp
+        The right-hand operator in the product.
+
+    Returns
+    -------
+    LinOp
+        A linear operator representing the Kronecker product.
+    """
+    return lo.LinOp(lo.KRON_L, shape, [lh_op], rh_op)
 
 
 def div_expr(lh_op, rh_op):
@@ -325,7 +350,7 @@ def div_expr(lh_op, rh_op):
     return lo.LinOp(lo.DIV, lh_op.shape, [lh_op], rh_op)
 
 
-def promote(operator, shape):
+def promote(operator, shape: Tuple[int, ...]):
     """Promotes a scalar operator to the given shape.
 
     Parameters
@@ -343,7 +368,7 @@ def promote(operator, shape):
     return lo.LinOp(lo.PROMOTE, shape, [operator], None)
 
 
-def sum_entries(operator, shape):
+def sum_entries(operator, shape: Tuple[int, ...]):
     """Sum the entries of an operator.
 
     Parameters
@@ -377,7 +402,7 @@ def trace(operator):
     return lo.LinOp(lo.TRACE, (1, 1), [operator], None)
 
 
-def index(operator, shape, keys):
+def index(operator, shape: Tuple[int, ...], keys):
     """Indexes/slices an operator.
 
     Parameters
@@ -397,7 +422,7 @@ def index(operator, shape, keys):
     return lo.LinOp(lo.INDEX, shape, [operator], keys)
 
 
-def conv(lh_op, rh_op, shape):
+def conv(lh_op, rh_op, shape: Tuple[int, ...]):
     """1D discrete convolution of two vectors.
 
     Parameters
@@ -433,13 +458,13 @@ def transpose(operator):
     if len(operator.shape) < 2:
         return operator
     elif len(operator.shape) > 2:
-        return NotImplemented
+        raise NotImplementedError()
     else:
         shape = (operator.shape[1], operator.shape[0])
         return lo.LinOp(lo.TRANSPOSE, shape, [operator], None)
 
 
-def reshape(operator, shape):
+def reshape(operator, shape: Tuple[int, ...]):
     """Reshapes an operator.
 
     Parameters
@@ -509,7 +534,7 @@ def upper_tri(operator):
     return lo.LinOp(lo.UPPER_TRI, shape, [operator], None)
 
 
-def hstack(operators, shape):
+def hstack(operators, shape: Tuple[int, ...]):
     """Concatenates operators horizontally.
 
     Parameters
@@ -527,7 +552,7 @@ def hstack(operators, shape):
     return lo.LinOp(lo.HSTACK, shape, operators, None)
 
 
-def vstack(operators, shape):
+def vstack(operators, shape: Tuple[int, ...]):
     """Concatenates operators vertically.
 
     Parameters
