@@ -17,15 +17,21 @@ limitations under the License.
 from cvxpy.atoms import sum_squares
 from cvxpy.atoms.quad_form import decomp_quad
 from cvxpy.expressions.constants import Constant
-from cvxpy.reductions.dcp2cone.atom_canonicalizers.quad_over_lin_canon import quad_over_lin_canon
+from cvxpy.reductions.dcp2cone.atom_canonicalizers.quad_over_lin_canon import (
+    quad_over_lin_canon,)
 
 
 def quad_form_canon(expr, args):
+    # TODO this doesn't work with parameters!
     scale, M1, M2 = decomp_quad(args[1].value)
+    # Special case where P == 0.
+    if M1.size == M2.size == 0:
+        return Constant(0), []
+
     if M1.size > 0:
-        expr = sum_squares(Constant(M1.T) * args[0])
+        expr = sum_squares(Constant(M1.T) @ args[0])
     if M2.size > 0:
         scale = -scale
-        expr = sum_squares(Constant(M2.T) * args[0])
+        expr = sum_squares(Constant(M2.T) @ args[0])
     obj, constr = quad_over_lin_canon(expr, expr.args)
     return scale * obj, constr
