@@ -15,7 +15,8 @@ limitations under the License.
 """
 
 from cvxpy.atoms.elementwise.maximum import maximum
-from cvxpy.reductions.eliminate_pwl.atom_canonicalizers.maximum_canon import maximum_canon
+from cvxpy.reductions.eliminate_pwl.atom_canonicalizers.maximum_canon import (
+    maximum_canon,)
 
 
 def minimum_canon(expr, args):
