@@ -17,12 +17,16 @@ limitations under the License.
 from .cbc_conif import CBC
 from .cplex_conif import CPLEX
 from .cvxopt_conif import CVXOPT
+from .diffcp_conif import DIFFCP
 from .ecos_conif import ECOS
+from .glop_conif import GLOP
 from .glpk_conif import GLPK
 from .glpk_mi_conif import GLPK_MI
 from .gurobi_conif import GUROBI
 from .mosek_conif import MOSEK
-from .scs_conif import SCS
-from .super_scs_conif import SuperSCS
-from .xpress_conif import XPRESS
 from .nag_conif import NAG
+from .pdlp_conif import PDLP
+from .scip_conif import SCIP
+from .scipy_conif import SCIPY
+from .scs_conif import SCS
+from .xpress_conif import XPRESS
