@@ -14,16 +14,18 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+from collections import namedtuple
+from operator import attrgetter
+
+import numpy as np
+from scipy.sparse import dok_matrix
+
 import cvxpy.settings as s
 from cvxpy.constraints import SOC
-from cvxpy.reductions.solvers.conic_solvers.scs_conif import (SCS,
-                                                              dims_to_solver_dict)
-from cvxpy.reductions.solvers.conic_solvers.conic_solver import ConicSolver
 from cvxpy.reductions.solution import Solution, failure_solution
 from cvxpy.reductions.solvers import utilities
-from collections import namedtuple
-from scipy.sparse import dok_matrix
-import numpy as np
+from cvxpy.reductions.solvers.conic_solvers.conic_solver import (
+    ConicSolver, dims_to_solver_dict,)
 
 # Values used to distinguish between linear and quadratic constraints.
 _LIN, _QUAD = 0, 1
@@ -34,8 +36,174 @@ _LIN, _QUAD = 0, 1
 _CpxConstr = namedtuple("_CpxConstr", ["constr_type", "index"])
 
 
-class CPLEX(SCS):
-    """An interface for the CPLEX solver."""
+def set_parameters(model, solver_opts) -> None:
+    """Sets CPLEX parameters."""
+    kwargs = sorted(solver_opts.keys())
+    if "cplex_params" in kwargs:
+        for param, value in solver_opts["cplex_params"].items():
+            try:
+                f = attrgetter(param)
+                parameter = f(model.parameters)
+                parameter.set(value)
+            except AttributeError:
+                raise ValueError(
+                    "invalid CPLEX parameter, value pair ({0}, {1})".format(
+                        param, value))
+        kwargs.remove("cplex_params")
+    if "cplex_filename" in kwargs:
+        filename = solver_opts["cplex_filename"]
+        if filename:
+            model.write(filename)
+        kwargs.remove("cplex_filename")
+    if kwargs:
+        raise ValueError("invalid keyword-argument '{0}'".format(kwargs[0]))
+
+
+def hide_solver_output(model) -> None:
+    """Set CPLEX verbosity level (either on or off)."""
+    # By default the output will be sent to stdout. Setting the output
+    # streams to None will prevent any output from being shown.
+    model.set_results_stream(None)
+    model.set_warning_stream(None)
+    model.set_error_stream(None)
+    model.set_log_stream(None)
+
+
+def _handle_solve_status(model, solstat):
+    """Map CPLEX MIP solution status codes to non-MIP status codes."""
+    status = model.solution.status
+
+    # With CPLEX 12.10, some benders status codes were removed.
+    if model.get_versionnumber() < 12100000:
+        unbounded_status_codes = (
+            status.MIP_unbounded,
+            status.MIP_benders_master_unbounded,
+            status.benders_master_unbounded
+        )
+    else:
+        unbounded_status_codes = (
+            status.MIP_unbounded,
+        )
+
+    if solstat == status.MIP_optimal:
+        return status.optimal
+    elif solstat == status.MIP_infeasible:
+        return status.infeasible
+    elif solstat in (status.MIP_time_limit_feasible,
+                     status.MIP_time_limit_infeasible):
+        return status.abort_time_limit
+    elif solstat in (status.MIP_dettime_limit_feasible,
+                     status.MIP_dettime_limit_infeasible):
+        return status.abort_dettime_limit
+    elif solstat in (status.MIP_abort_feasible,
+                     status.MIP_abort_infeasible):
+        return status.abort_user
+    elif solstat == status.MIP_optimal_infeasible:
+        return status.optimal_infeasible
+    elif solstat == status.MIP_infeasible_or_unbounded:
+        return status.infeasible_or_unbounded
+    elif solstat in unbounded_status_codes:
+        return status.unbounded
+    elif solstat in (status.feasible_relaxed_sum,
+                     status.MIP_feasible_relaxed_sum,
+                     status.optimal_relaxed_sum,
+                     status.MIP_optimal_relaxed_sum,
+                     status.feasible_relaxed_inf,
+                     status.MIP_feasible_relaxed_inf,
+                     status.optimal_relaxed_inf,
+                     status.MIP_optimal_relaxed_inf,
+                     status.feasible_relaxed_quad,
+                     status.MIP_feasible_relaxed_quad,
+                     status.optimal_relaxed_quad,
+                     status.MIP_optimal_relaxed_quad):
+        raise AssertionError(
+            "feasopt status encountered: {0}".format(solstat))
+    elif solstat in (status.conflict_feasible,
+                     status.conflict_minimal,
+                     status.conflict_abort_contradiction,
+                     status.conflict_abort_time_limit,
+                     status.conflict_abort_dettime_limit,
+                     status.conflict_abort_iteration_limit,
+                     status.conflict_abort_node_limit,
+                     status.conflict_abort_obj_limit,
+                     status.conflict_abort_memory_limit,
+                     status.conflict_abort_user):
+        raise AssertionError(
+            "conflict refiner status encountered: {0}".format(solstat))
+    elif solstat == status.relaxation_unbounded:
+        return status.relaxation_unbounded
+    elif solstat in (status.feasible,
+                     status.MIP_feasible):
+        return status.feasible
+    elif solstat == status.benders_num_best:
+        return status.num_best
+    else:
+        return solstat
+
+
+def get_status(model):
+    """Map CPLEX status to CPXPY status."""
+    pfeas = model.solution.is_primal_feasible()
+    # NOTE: dfeas is always false for a MIP.
+    dfeas = model.solution.is_dual_feasible()
+    status = model.solution.status
+    solstat = _handle_solve_status(model, model.solution.get_status())
+    if solstat in (status.node_limit_infeasible,
+                   status.fail_infeasible,
+                   status.mem_limit_infeasible,
+                   status.fail_infeasible_no_tree,
+                   status.num_best):
+        return s.SOLVER_ERROR
+    elif solstat in (status.abort_user,
+                     status.abort_iteration_limit,
+                     status.abort_time_limit,
+                     status.abort_dettime_limit,
+                     status.abort_obj_limit,
+                     status.abort_primal_obj_limit,
+                     status.abort_dual_obj_limit,
+                     status.abort_relaxed,
+                     status.first_order):
+        if pfeas:
+            return s.OPTIMAL_INACCURATE
+        else:
+            return s.SOLVER_ERROR
+    elif solstat in (status.node_limit_feasible,
+                     status.solution_limit,
+                     status.populate_solution_limit,
+                     status.fail_feasible,
+                     status.mem_limit_feasible,
+                     status.fail_feasible_no_tree,
+                     status.feasible):
+        if dfeas:
+            return s.OPTIMAL
+        else:
+            return s.OPTIMAL_INACCURATE
+    elif solstat in (status.optimal,
+                     status.optimal_tolerance,
+                     status.optimal_infeasible,
+                     status.optimal_populated,
+                     status.optimal_populated_tolerance):
+        return s.OPTIMAL
+    elif solstat in (status.infeasible,
+                     status.optimal_relaxed_sum,
+                     status.optimal_relaxed_inf,
+                     status.optimal_relaxed_quad):
+        return s.INFEASIBLE
+    elif solstat in (status.feasible_relaxed_quad,
+                     status.feasible_relaxed_inf,
+                     status.feasible_relaxed_sum):
+        return s.SOLVER_ERROR
+    elif solstat == status.unbounded:
+        return s.UNBOUNDED
+    elif solstat == status.infeasible_or_unbounded:
+        return s.INFEASIBLE_OR_UNBOUNDED
+    else:
+        return s.SOLVER_ERROR
+
+
+class CPLEX(ConicSolver):
+    """An interface for the CPLEX solver.
+    """
 
     # Solver capabilities.
     MIP_CAPABLE = True
@@ -45,12 +213,12 @@ class CPLEX(SCS):
         """The name of the solver. """
         return s.CPLEX
 
-    def import_solver(self):
+    def import_solver(self) -> None:
         """Imports the solver."""
         import cplex
         cplex  # For flake8
 
-    def accepts(self, problem):
+    def accepts(self, problem) -> bool:
         """Can CPLEX solve the problem?
         """
         # TODO check if is matrix stuffed.
@@ -73,7 +241,7 @@ class CPLEX(SCS):
             (dict of arguments needed for the solver, inverse data)
         """
         data, inv_data = super(CPLEX, self).apply(problem)
-        variables = problem.variables()[0]
+        variables = problem.x
         data[s.BOOL_IDX] = [int(t[0]) for t in variables.boolean_idx]
         data[s.INT_IDX] = [int(t[0]) for t in variables.integer_idx]
         inv_data['is_mip'] = data[s.BOOL_IDX] or data[s.INT_IDX]
@@ -83,29 +251,38 @@ class CPLEX(SCS):
     def invert(self, solution, inverse_data):
         """Returns the solution to the original problem given the inverse_data.
         """
-        status = solution['status']
+        model = solution["model"]
+        attr = {}
+        if s.SOLVE_TIME in solution:
+            attr[s.SOLVE_TIME] = solution[s.SOLVE_TIME]
+
+        status = get_status(model)
 
         primal_vars = None
         dual_vars = None
         if status in s.SOLUTION_PRESENT:
-            opt_val = solution['value'] + inverse_data[s.OFFSET]
-            primal_vars = {inverse_data[CPLEX.VAR_ID]: solution['primal']}
+            opt_val = (model.solution.get_objective_value() +
+                       inverse_data[s.OFFSET])
+
+            x = np.array(model.solution.get_values())
+            primal_vars = {inverse_data[CPLEX.VAR_ID]: x}
+
             if not inverse_data['is_mip']:
-                eq_dual = utilities.get_dual_values(
-                    solution['eq_dual'],
+                # The dual values are retrieved in the order that the
+                # constraints were added in solve_via_data() below. We
+                # must be careful to map them to inverse_data[EQ_CONSTR]
+                # followed by inverse_data[NEQ_CONSTR] accordingly.
+                y = -np.array(model.solution.get_dual_values())
+                dual_vars = utilities.get_dual_values(
+                    y,
                     utilities.extract_dual_value,
-                    inverse_data[CPLEX.EQ_CONSTR])
-                leq_dual = utilities.get_dual_values(
-                    solution['ineq_dual'],
-                    utilities.extract_dual_value,
-                    inverse_data[CPLEX.NEQ_CONSTR])
-                eq_dual.update(leq_dual)
-                dual_vars = eq_dual
-            return Solution(status, opt_val, primal_vars, dual_vars, {})
+                    inverse_data[CPLEX.EQ_CONSTR] + inverse_data[CPLEX.NEQ_CONSTR])
+
+            return Solution(status, opt_val, primal_vars, dual_vars, attr)
         else:
             return failure_solution(status)
 
-    def solve_via_data(self, data, warm_start, verbose, solver_opts, solver_cache=None):
+    def solve_via_data(self, data, warm_start: bool, verbose: bool, solver_opts, solver_cache=None):
         import cplex
 
         c = data[s.C]
@@ -148,7 +325,7 @@ class CPLEX(SCS):
         cpx_constrs += [_CpxConstr(_LIN, x)
                         for x in self.add_model_lin_constr(
                                 model, variables,
-                                list(range(dims[s.EQ_DIM])),
+                                range(dims[s.EQ_DIM]),
                                 'E', A, b)]
 
         # Add inequality (<=) constraints
@@ -157,7 +334,7 @@ class CPLEX(SCS):
         cpx_constrs += [_CpxConstr(_LIN, x)
                         for x in self.add_model_lin_constr(
                                 model, variables,
-                                list(range(leq_start, leq_end)),
+                                range(leq_start, leq_end),
                                 'L', A, b)]
 
         # Add SOC constraints
@@ -165,7 +342,7 @@ class CPLEX(SCS):
         for constr_len in dims[s.SOC_DIM]:
             soc_end = soc_start + constr_len
             soc_constr, new_leq, new_vars = self.add_model_soc_constr(
-                model, variables, list(range(soc_start, soc_end)), A, b)
+                model, variables, range(soc_start, soc_end), A, b)
             cpx_constrs.append(_CpxConstr(_QUAD, soc_constr))
             cpx_constrs += [_CpxConstr(_LIN, x) for x in new_leq]
             variables += new_vars
@@ -173,193 +350,37 @@ class CPLEX(SCS):
 
         # Set verbosity
         if not verbose:
-            model.set_results_stream(None)
-            model.set_warning_stream(None)
-            model.set_error_stream(None)
-            model.set_log_stream(None)
-        else:
-            # By default the output will be sent to stdout.
-            pass
+            hide_solver_output(model)
 
-        # TODO: user option to not compute duals.
+        # For CVXPY, we set the qcpduals parameter here, but the user can
+        # easily override it via the "cplex_params" solver option (see
+        # set_parameters function).
         model.parameters.preprocessing.qcpduals.set(
             model.parameters.preprocessing.qcpduals.values.force)
 
-        # TODO: Parameter support is functional, but perhaps not ideal.
-        # The user must pass parameter names as used in the CPLEX Python
-        # API, and raw values (i.e., no enum support).
-        kwargs = sorted(solver_opts.keys())
-        if "cplex_params" in kwargs:
-            for param, value in list(solver_opts["cplex_params"].items()):
-                try:
-                    eval("model.parameters.{0}.set({1})".format(param, value))
-                except AttributeError:
-                    raise ValueError(
-                        "invalid CPLEX parameter, value pair ({0}, {1})".format(
-                            param, value))
-            kwargs.remove("cplex_params")
-        if "cplex_filename" in kwargs:
-            filename = solver_opts["cplex_filename"]
-            if filename:
-                model.write(filename)
-            kwargs.remove("cplex_filename")
-        if s.BOOL_IDX in kwargs:
-            kwargs.remove(s.BOOL_IDX)
-        if s.INT_IDX in kwargs:
-            kwargs.remove(s.INT_IDX)
-        if kwargs:
-            raise ValueError("invalid keyword-argument '{0}'".format(kwargs[0]))
+        # Set parameters
+        reoptimize = solver_opts.pop('reoptimize', False)
+        set_parameters(model, solver_opts)
 
-        solution = {}
-        start_time = model.get_time()
-        solution[s.SOLVE_TIME] = -1
+        # Solve problem
+        solution = {"model": model}
         try:
+            start_time = model.get_time()
             model.solve()
             solution[s.SOLVE_TIME] = model.get_time() - start_time
-            solution["value"] = model.solution.get_objective_value()
-            solution["primal"] = np.array(model.solution.get_values(variables))
-            solution["status"] = self._get_status(model)
 
-            # Only add duals if not a MIP.
-            if not (data[s.BOOL_IDX] or data[s.INT_IDX]):
-                vals = []
-                for con in cpx_constrs:
-                    assert con.index is not None
-                    if con.constr_type == _LIN:
-                        vals.append(model.solution.get_dual_values(con.index))
-                    else:
-                        assert con.constr_type == _QUAD
-                        # Quadratic constraints not queried directly.
-                        vals.append(0.0)
-                solution["y"] = -np.array(vals)
-                solution[s.EQ_DUAL] = solution["y"][0:dims[s.EQ_DIM]]
-                solution[s.INEQ_DUAL] = solution["y"][dims[s.EQ_DIM]:]
+            ambiguous_status = get_status(model) == s.INFEASIBLE_OR_UNBOUNDED
+            if ambiguous_status and reoptimize:
+                model.parameters.preprocessing.presolve.set(0)
+                start_time = model.get_time()
+                model.solve()
+                solution[s.SOLVE_TIME] += model.get_time() - start_time
+
         except Exception:
-            if solution[s.SOLVE_TIME] < 0.0:
-                solution[s.SOLVE_TIME] = model.get_time() - start_time
-            solution["status"] = s.SOLVER_ERROR
+            pass
 
         return solution
 
-    def _handle_solve_status(self, model, solstat):
-        """Map CPLEX MIP solution status codes to non-MIP status codes."""
-        status = model.solution.status
-        if solstat == status.MIP_optimal:
-            return status.optimal
-        elif solstat == status.MIP_infeasible:
-            return status.infeasible
-        elif solstat in (status.MIP_time_limit_feasible,
-                         status.MIP_time_limit_infeasible):
-            return status.abort_time_limit
-        elif solstat in (status.MIP_dettime_limit_feasible,
-                         status.MIP_dettime_limit_infeasible):
-            return status.abort_dettime_limit
-        elif solstat in (status.MIP_abort_feasible,
-                         status.MIP_abort_infeasible):
-            return status.abort_user
-        elif solstat == status.MIP_optimal_infeasible:
-            return status.optimal_infeasible
-        elif solstat == status.MIP_infeasible_or_unbounded:
-            return status.infeasible_or_unbounded
-        elif solstat in (status.MIP_unbounded,
-                         status.MIP_benders_master_unbounded,
-                         status.benders_master_unbounded):
-            return status.unbounded
-        elif solstat in (status.feasible_relaxed_sum,
-                         status.MIP_feasible_relaxed_sum,
-                         status.optimal_relaxed_sum,
-                         status.MIP_optimal_relaxed_sum,
-                         status.feasible_relaxed_inf,
-                         status.MIP_feasible_relaxed_inf,
-                         status.optimal_relaxed_inf,
-                         status.MIP_optimal_relaxed_inf,
-                         status.feasible_relaxed_quad,
-                         status.MIP_feasible_relaxed_quad,
-                         status.optimal_relaxed_quad,
-                         status.MIP_optimal_relaxed_quad):
-            raise AssertionError(
-                "feasopt status encountered: {0}".format(solstat))
-        elif solstat in (status.conflict_feasible,
-                         status.conflict_minimal,
-                         status.conflict_abort_contradiction,
-                         status.conflict_abort_time_limit,
-                         status.conflict_abort_dettime_limit,
-                         status.conflict_abort_iteration_limit,
-                         status.conflict_abort_node_limit,
-                         status.conflict_abort_obj_limit,
-                         status.conflict_abort_memory_limit,
-                         status.conflict_abort_user):
-            raise AssertionError(
-                "conflict refiner status encountered: {0}".format(solstat))
-        elif solstat == status.relaxation_unbounded:
-            return status.relaxation_unbounded
-        elif solstat in (status.feasible,
-                         status.MIP_feasible):
-            return status.feasible
-        elif solstat == status.benders_num_best:
-            return status.num_best
-        else:
-            return solstat
-
-    def _get_status(self, model):
-        """Map CPLEX status to CPXPY status."""
-        pfeas = model.solution.is_primal_feasible()
-        # NOTE: dfeas is always false for a MIP.
-        dfeas = model.solution.is_dual_feasible()
-        status = model.solution.status
-        solstat = self._handle_solve_status(model, model.solution.get_status())
-        if solstat in (status.node_limit_infeasible,
-                       status.fail_infeasible,
-                       status.mem_limit_infeasible,
-                       status.fail_infeasible_no_tree,
-                       status.num_best):
-            return s.SOLVER_ERROR
-        elif solstat in (status.abort_user,
-                         status.abort_iteration_limit,
-                         status.abort_time_limit,
-                         status.abort_dettime_limit,
-                         status.abort_obj_limit,
-                         status.abort_primal_obj_limit,
-                         status.abort_dual_obj_limit,
-                         status.abort_relaxed,
-                         status.first_order):
-            if pfeas:
-                return s.OPTIMAL_INACCURATE
-            else:
-                return s.SOLVER_ERROR
-        elif solstat in (status.node_limit_feasible,
-                         status.solution_limit,
-                         status.populate_solution_limit,
-                         status.fail_feasible,
-                         status.mem_limit_feasible,
-                         status.fail_feasible_no_tree,
-                         status.feasible):
-            if dfeas:
-                return s.OPTIMAL
-            else:
-                return s.OPTIMAL_INACCURATE
-        elif solstat in (status.optimal,
-                         status.optimal_tolerance,
-                         status.optimal_infeasible,
-                         status.optimal_populated,
-                         status.optimal_populated_tolerance):
-            return s.OPTIMAL
-        elif solstat in (status.infeasible,
-                         status.optimal_relaxed_sum,
-                         status.optimal_relaxed_inf,
-                         status.optimal_relaxed_quad):
-            return s.INFEASIBLE
-        elif solstat in (status.feasible_relaxed_quad,
-                         status.feasible_relaxed_inf,
-                         status.feasible_relaxed_sum):
-            return s.SOLVER_ERROR
-        elif solstat == status.infeasible_or_unbounded:
-            return s.INFEASIBLE
-        elif solstat == status.unbounded:
-            return s.UNBOUNDED
-        else:
-            return s.SOLVER_ERROR
-
     def add_model_lin_constr(self, model, variables,
                              rows, ctype, mat, vec):
         """Adds EQ/LEQ constraints to the model using the data from mat and vec.
@@ -391,7 +412,7 @@ class CPLEX(SCS):
             val = [x for x in csr[i].data]
             lin_expr.append([ind, val])
             rhs.append(vec[i])
-        # For better performance, we add the contraints in a batch.
+        # For better performance, we add the constraints in a batch.
         if lin_expr:
             assert len(lin_expr) == len(rhs)
             constr.extend(list(
@@ -426,6 +447,7 @@ class CPLEX(SCS):
             supporting variable indices).
         """
         import cplex
+
         # Assume first expression (i.e. t) is nonzero.
         lin_expr_list, soc_vars, lin_rhs = [], [], []
         csr = mat.tocsr()
@@ -435,10 +457,9 @@ class CPLEX(SCS):
             # Ignore empty constraints.
             if ind:
                 lin_expr_list.append((ind, val))
-                lin_rhs.append(vec[i])
             else:
                 lin_expr_list.append(None)
-                lin_rhs.append(0.0)
+            lin_rhs.append(vec[i])
 
         # Make a variable and equality constraint for each term.
         soc_vars, is_first = [], True
