@@ -1,15 +1,18 @@
+import numpy as np
+
 import cvxpy
-from cvxpy.atoms.affine.add_expr import AddExpression
 import cvxpy.error as error
 import cvxpy.reductions.dgp2dcp.atom_canonicalizers as dgp_atom_canon
+from cvxpy.atoms.affine.add_expr import AddExpression
 from cvxpy.reductions import solution
 from cvxpy.settings import SOLVER_ERROR
 from cvxpy.tests.base_test import BaseTest
-import numpy as np
+
+SOLVER = cvxpy.ECOS
 
 
 class TestDgp2Dcp(BaseTest):
-    def test_unconstrained_monomial(self):
+    def test_unconstrained_monomial(self) -> None:
         x = cvxpy.Variable(pos=True)
         y = cvxpy.Variable(pos=True)
         prod = x * y
@@ -21,7 +24,7 @@ class TestDgp2Dcp(BaseTest):
         self.assertEqual(len(dcp.objective.expr.args), 2)
         self.assertIsInstance(dcp.objective.expr.args[0], cvxpy.Variable)
         self.assertIsInstance(dcp.objective.expr.args[1], cvxpy.Variable)
-        opt = dcp.solve()
+        opt = dcp.solve(SOLVER)
         # dcp is solved in log-space, so it is unbounded below
         # (since the OPT for dgp is 0 + epsilon).
         self.assertEqual(opt, -float("inf"))
@@ -31,32 +34,32 @@ class TestDgp2Dcp(BaseTest):
         self.assertAlmostEqual(dgp.value, 0.0)
         self.assertEqual(dgp.status, "unbounded")
         dgp._clear_solution()
-        dgp.solve(gp=True)
+        dgp.solve(SOLVER, gp=True)
         self.assertAlmostEqual(dgp.value, 0.0)
         self.assertEqual(dgp.status, "unbounded")
 
         dgp = cvxpy.Problem(cvxpy.Maximize(prod), [])
         dgp2dcp = cvxpy.reductions.Dgp2Dcp(dgp)
         dcp = dgp2dcp.reduce()
-        self.assertEqual(dcp.solve(), float("inf"))
+        self.assertEqual(dcp.solve(SOLVER), float("inf"))
         self.assertEqual(dcp.status, "unbounded")
 
         dgp.unpack(dgp2dcp.retrieve(dcp.solution))
         self.assertEqual(dgp.value, float("inf"))
         self.assertEqual(dgp.status, "unbounded")
         dgp._clear_solution()
-        dgp.solve(gp=True)
+        dgp.solve(SOLVER, gp=True)
         self.assertAlmostEqual(dgp.value, float("inf"))
         self.assertEqual(dgp.status, "unbounded")
 
-    def test_basic_equality_constraint(self):
+    def test_basic_equality_constraint(self) -> None:
         x = cvxpy.Variable(pos=True)
         dgp = cvxpy.Problem(cvxpy.Minimize(x), [x == 1.0])
         dgp2dcp = cvxpy.reductions.Dgp2Dcp(dgp)
 
         dcp = dgp2dcp.reduce()
         self.assertIsInstance(dcp.objective.expr, cvxpy.Variable)
-        opt = dcp.solve()
+        opt = dcp.solve(SOLVER)
         self.assertAlmostEqual(opt, 0.0)
         self.assertAlmostEqual(dcp.variables()[0].value, 0.0)
 
@@ -64,18 +67,18 @@ class TestDgp2Dcp(BaseTest):
         self.assertAlmostEqual(dgp.value, 1.0)
         self.assertAlmostEqual(x.value, 1.0)
         dgp._clear_solution()
-        dgp.solve(gp=True)
+        dgp.solve(SOLVER, gp=True)
         self.assertAlmostEqual(dgp.value, 1.0)
         self.assertAlmostEqual(x.value, 1.0)
 
-    def test_basic_gp(self):
+    def test_basic_gp(self) -> None:
         x, y, z = cvxpy.Variable((3,), pos=True)
         constraints = [2*x*y + 2*x*z + 2*y*z <= 1.0, x >= 2*y]
         problem = cvxpy.Problem(cvxpy.Minimize(1/(x*y*z)), constraints)
-        problem.solve(gp=True)
+        problem.solve(SOLVER, gp=True)
         self.assertAlmostEqual(15.59, problem.value, places=2)
 
-    def test_maximum(self):
+    def test_maximum(self) -> None:
         x = cvxpy.Variable(pos=True)
         y = cvxpy.Variable(pos=True)
 
@@ -87,17 +90,17 @@ class TestDgp2Dcp(BaseTest):
         dgp = cvxpy.Problem(obj, constr)
         dgp2dcp = cvxpy.reductions.Dgp2Dcp(dgp)
         dcp = dgp2dcp.reduce()
-        dcp.solve()
+        dcp.solve(SOLVER)
         dgp.unpack(dgp2dcp.retrieve(dcp.solution))
         self.assertAlmostEqual(dgp.value, 6.0)
         self.assertAlmostEqual(x.value, 1.0)
         self.assertAlmostEqual(y.value, 4.0)
         dgp._clear_solution()
-        dgp.solve(gp=True)
-        self.assertAlmostEqual(dgp.value, 6.0)
+        dgp.solve(SOLVER, gp=True)
+        self.assertAlmostEqual(dgp.value, 6.0, places=4)
         self.assertAlmostEqual(x.value, 1.0)
 
-    def test_prod(self):
+    def test_prod(self) -> None:
         X = np.arange(12).reshape((4, 3))
         np.testing.assert_almost_equal(np.prod(X), cvxpy.prod(X).value)
         np.testing.assert_almost_equal(
@@ -155,7 +158,7 @@ class TestDgp2Dcp(BaseTest):
         self.assertTrue(cvxpy.prod([m, 1/posy1]).is_log_log_concave())
         self.assertFalse(cvxpy.prod([m, 1/posy1]).is_log_log_convex())
 
-    def test_max(self):
+    def test_max(self) -> None:
         x = cvxpy.Variable(pos=True)
         y = cvxpy.Variable(pos=True)
 
@@ -167,17 +170,17 @@ class TestDgp2Dcp(BaseTest):
         dgp = cvxpy.Problem(obj, constr)
         dgp2dcp = cvxpy.reductions.Dgp2Dcp(dgp)
         dcp = dgp2dcp.reduce()
-        dcp.solve()
+        dcp.solve(SOLVER)
         dgp.unpack(dgp2dcp.retrieve(dcp.solution))
         self.assertAlmostEqual(dgp.value, 6.0)
         self.assertAlmostEqual(x.value, 1.0)
         self.assertAlmostEqual(y.value, 4.0)
         dgp._clear_solution()
-        dgp.solve(gp=True)
-        self.assertAlmostEqual(dgp.value, 6.0)
+        dgp.solve(SOLVER, gp=True)
+        self.assertAlmostEqual(dgp.value, 6.0, places=4)
         self.assertAlmostEqual(x.value, 1.0)
 
-    def test_minimum(self):
+    def test_minimum(self) -> None:
         x = cvxpy.Variable(pos=True)
         y = cvxpy.Variable(pos=True)
 
@@ -188,12 +191,12 @@ class TestDgp2Dcp(BaseTest):
         constr = [x == 1.0, y == 4.0]
 
         dgp = cvxpy.Problem(obj, constr)
-        dgp.solve(gp=True)
+        dgp.solve(SOLVER, gp=True)
         self.assertAlmostEqual(dgp.value, 1.0 / (2.0 + 6.0))
         self.assertAlmostEqual(x.value, 1.0)
         self.assertAlmostEqual(y.value, 4.0)
 
-    def test_min(self):
+    def test_min(self) -> None:
         x = cvxpy.Variable(pos=True)
         y = cvxpy.Variable(pos=True)
 
@@ -204,12 +207,12 @@ class TestDgp2Dcp(BaseTest):
         constr = [x == 1.0, y == 4.0]
 
         dgp = cvxpy.Problem(obj, constr)
-        dgp.solve(gp=True)
-        self.assertAlmostEqual(dgp.value, 1.0 / (2.0 + 6.0))
+        dgp.solve(SOLVER, gp=True)
+        self.assertAlmostEqual(dgp.value, 1.0 / (2.0 + 6.0), places=4)
         self.assertAlmostEqual(x.value, 1.0)
         self.assertAlmostEqual(y.value, 4.0)
 
-    def test_sum_largest(self):
+    def test_sum_largest(self) -> None:
         self.skipTest("Enable test once sum_largest is implemented.")
         x = cvxpy.Variable((4,), pos=True)
         obj = cvxpy.Minimize(cvxpy.sum_largest(x, 3))
@@ -217,14 +220,14 @@ class TestDgp2Dcp(BaseTest):
         dgp = cvxpy.Problem(obj, constr)
         dgp2dcp = cvxpy.reductions.Dgp2Dcp(dgp)
         dcp = dgp2dcp.reduce()
-        dcp.solve()
+        dcp.solve(SOLVER)
         dgp.unpack(dgp2dcp.retrieve(dcp.solution))
         opt = 6.0
         self.assertAlmostEqual(dgp.value, opt)
         self.assertAlmostEqual((x[0] * x[1] * x[2] * x[3]).value, 16,
                                places=2)
         dgp._clear_solution()
-        dgp.solve(gp=True)
+        dgp.solve(SOLVER, gp=True)
         self.assertAlmostEqual(dgp.value, opt)
         self.assertAlmostEqual((x[0] * x[1] * x[2] * x[3]).value, 16,
                                places=2)
@@ -237,13 +240,13 @@ class TestDgp2Dcp(BaseTest):
         dgp = cvxpy.Problem(obj, constr)
         dgp2dcp = cvxpy.reductions.Dgp2Dcp(dgp)
         dcp = dgp2dcp.reduce()
-        opt = dcp.solve()
+        opt = dcp.solve(SOLVER)
         self.assertEqual(dcp.value, -float("inf"))
         dgp.unpack(dgp2dcp.retrieve(dcp.solution))
         self.assertAlmostEqual(dgp.value, 0.0)
         self.assertAlmostEqual(dgp.status, "unbounded")
         dgp._clear_solution()
-        dgp.solve(gp=True)
+        dgp.solve(SOLVER, gp=True)
         self.assertAlmostEqual(dgp.value, 0.0)
         self.assertAlmostEqual(dgp.status, "unbounded")
 
@@ -253,13 +256,13 @@ class TestDgp2Dcp(BaseTest):
         dgp = cvxpy.Problem(obj, [])
         dgp2dcp = cvxpy.reductions.Dgp2Dcp(dgp)
         dcp = dgp2dcp.reduce()
-        opt = dcp.solve()
+        opt = dcp.solve(SOLVER)
         self.assertEqual(dcp.value, -float("inf"))
         dgp.unpack(dgp2dcp.retrieve(dcp.solution))
         self.assertAlmostEqual(dgp.value, 0.0)
         self.assertAlmostEqual(dgp.status, "unbounded")
         dgp._clear_solution()
-        dgp.solve(gp=True)
+        dgp.solve(SOLVER, gp=True)
         self.assertAlmostEqual(dgp.value, 0.0)
         self.assertAlmostEqual(dgp.status, "unbounded")
 
@@ -272,7 +275,7 @@ class TestDgp2Dcp(BaseTest):
         dgp = cvxpy.Problem(obj, constr)
         dgp2dcp = cvxpy.reductions.Dgp2Dcp(dgp)
         dcp = dgp2dcp.reduce()
-        dcp.solve()
+        dcp.solve(SOLVER)
         dgp.unpack(dgp2dcp.retrieve(dcp.solution))
         # opt = 3 * sqrt(4) * sqrt(4) + (4 * 4 + 0.5 * 4 * epsilon) = 28
         opt = 28.0
@@ -280,69 +283,71 @@ class TestDgp2Dcp(BaseTest):
         self.assertAlmostEqual((x[0] * x[1]).value, 16.0, places=2)
         self.assertAlmostEqual(x[3].value, 0.0, places=2)
         dgp._clear_solution()
-        dgp.solve(gp=True)
+        dgp.solve(SOLVER, gp=True)
         self.assertAlmostEqual(dgp.value, opt, places=2)
         self.assertAlmostEqual((x[0] * x[1]).value, 16.0, places=2)
         self.assertAlmostEqual(x[3].value, 0.0, places=2)
 
-    def test_div(self):
+    def test_div(self) -> None:
         x = cvxpy.Variable(pos=True)
         y = cvxpy.Variable(pos=True)
         p = cvxpy.Problem(cvxpy.Minimize(x * y),
                           [y/3 <= x, y >= 1])
-        self.assertAlmostEqual(p.solve(gp=True), 1.0 / 3.0)
+        self.assertAlmostEqual(p.solve(SOLVER, gp=True), 1.0 / 3.0)
         self.assertAlmostEqual(y.value, 1.0)
         self.assertAlmostEqual(x.value, 1.0 / 3.0)
 
-    def test_geo_mean(self):
+    def test_geo_mean(self) -> None:
         x = cvxpy.Variable(3, pos=True)
         p = [1, 2, 0.5]
         geo_mean = cvxpy.geo_mean(x, p)
         dgp = cvxpy.Problem(cvxpy.Minimize(geo_mean), [])
         dgp2dcp = cvxpy.reductions.Dgp2Dcp(dgp)
         dcp = dgp2dcp.reduce()
-        dcp.solve()
+        dcp.solve(SOLVER)
         self.assertEqual(dcp.value, -float("inf"))
         dgp.unpack(dgp2dcp.retrieve(dcp.solution))
         self.assertEqual(dgp.value, 0.0)
         self.assertEqual(dgp.status, "unbounded")
         dgp._clear_solution()
-        dgp.solve(gp=True)
+        dgp.solve(SOLVER, gp=True)
         self.assertEqual(dgp.value, 0.0)
         self.assertEqual(dgp.status, "unbounded")
 
-    def test_solving_non_dgp_problem_raises_error(self):
+    def test_solving_non_dgp_problem_raises_error(self) -> None:
         problem = cvxpy.Problem(cvxpy.Minimize(-1.0 * cvxpy.Variable()), [])
-        with self.assertRaisesRegex(error.DGPError, r"Problem does not follow DGP "
-                                     "rules(?s)*.*However, the problem does follow DCP rules.*"):
-            problem.solve(gp=True)
-        problem.solve()
+        with self.assertRaisesRegex(error.DGPError,
+                                    r"Problem does not follow DGP "
+                                    "rules(?s)*.*However, the problem does follow DCP rules.*"):
+            problem.solve(SOLVER, gp=True)
+        problem.solve(SOLVER)
         self.assertEqual(problem.status, "unbounded")
         self.assertEqual(problem.value, -float("inf"))
 
-    def test_solving_non_dcp_problem_raises_error(self):
+    def test_solving_non_dcp_problem_raises_error(self) -> None:
         problem = cvxpy.Problem(
           cvxpy.Minimize(cvxpy.Variable(pos=True) * cvxpy.Variable(pos=True)),
         )
-        with self.assertRaisesRegex(error.DCPError, r"Problem does not follow DCP "
-                                     "rules(?s)*.*However, the problem does follow DGP rules.*"):
-            problem.solve()
-        problem.solve(gp=True)
+        with self.assertRaisesRegex(error.DCPError,
+                                    r"Problem does not follow DCP "
+                                    "rules(?s)*.*However, the problem does follow DGP rules.*"):
+            problem.solve(SOLVER)
+        problem.solve(SOLVER, gp=True)
         self.assertEqual(problem.status, "unbounded")
         self.assertAlmostEqual(problem.value, 0.0)
 
-    def test_solving_non_dcp_problems_raises_detailed_error(self):
+    def test_solving_non_dcp_problems_raises_detailed_error(self) -> None:
         x = cvxpy.Variable(3)
         problem = cvxpy.Problem(cvxpy.Minimize(cvxpy.sum(x) - cvxpy.sum_squares(x)))
         with self.assertRaisesRegex(error.DCPError, r"The objective is not DCP"):
-            problem.solve()
+            problem.solve(SOLVER)
 
         x = cvxpy.Variable(name='x')
         problem = cvxpy.Problem(cvxpy.Minimize(x), [x * x <= 5])
         with self.assertRaisesRegex(error.DCPError, r"The following constraints are not DCP"):
-            problem.solve()
+            problem.solve(SOLVER)
 
-    def test_add_canon(self):
+    def test_add_canon(self) -> None:
         X = cvxpy.Constant(np.array([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]]))
         Y = cvxpy.Constant(np.array([[2.0, 3.0, 4.0], [5.0, 6.0, 7.0]]))
         Z = X + Y
@@ -362,7 +367,7 @@ class TestDgp2Dcp(BaseTest):
         expected = np.log(np.exp(X.value) + np.exp(y.value))
         np.testing.assert_almost_equal(expected, canon_matrix.value)
 
-    def test_matmul_canon(self):
+    def test_matmul_canon(self) -> None:
         X = cvxpy.Constant(np.array([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]]))
         Y = cvxpy.Constant(np.array([[1.0], [2.0], [3.0]]))
         Z = cvxpy.matmul(X, Y)
@@ -375,7 +380,7 @@ class TestDgp2Dcp(BaseTest):
         self.assertAlmostEqual(first_entry, canon_matrix[0, 0].value)
         self.assertAlmostEqual(second_entry, canon_matrix[1, 0].value)
 
-    def test_trace_canon(self):
+    def test_trace_canon(self) -> None:
         X = cvxpy.Constant(np.array([[1.0, 5.0], [9.0, 14.0]]))
         Y = cvxpy.trace(X)
         canon, constraints = dgp_atom_canon.trace_canon(Y, Y.args)
@@ -384,16 +389,16 @@ class TestDgp2Dcp(BaseTest):
         expected = np.log(np.exp(1.0) + np.exp(14.0))
         self.assertAlmostEqual(expected, canon.value)
 
-    def test_one_minus_pos(self):
+    def test_one_minus_pos(self) -> None:
         x = cvxpy.Variable(pos=True)
         obj = cvxpy.Maximize(x)
         constr = [cvxpy.one_minus_pos(x) >= 0.4]
         problem = cvxpy.Problem(obj, constr)
-        problem.solve(gp=True)
+        problem.solve(SOLVER, gp=True)
         self.assertAlmostEqual(problem.value, 0.6)
         self.assertAlmostEqual(x.value, 0.6)
 
-    def test_qp_solver_not_allowed(self):
+    def test_qp_solver_not_allowed(self) -> None:
         x = cvxpy.Variable(pos=True)
         problem = cvxpy.Problem(cvxpy.Minimize(x))
         error_msg = ("When `gp=True`, `solver` must be a conic solver "
@@ -403,7 +408,7 @@ class TestDgp2Dcp(BaseTest):
             problem.solve(solver="OSQP", gp=True)
             self.assertEqual(error_msg, str(err))
 
-    def test_paper_example_sum_largest(self):
+    def test_paper_example_sum_largest(self) -> None:
         self.skipTest("Enable test once sum_largest is implemented.")
         x = cvxpy.Variable((4,), pos=True)
         x0, x1, x2, x3 = (x[0], x[1], x[2], x[3])
@@ -415,35 +420,47 @@ class TestDgp2Dcp(BaseTest):
         constr = [x0 * x1 * x2 >= 16]
         p = cvxpy.Problem(obj, constr)
         # smoke test.
-        p.solve(gp=True)
+        p.solve(SOLVER, gp=True)
 
-    def test_paper_example_one_minus_pos(self):
+    def test_paper_example_one_minus_pos(self) -> None:
         x = cvxpy.Variable(pos=True)
         y = cvxpy.Variable(pos=True)
         obj = cvxpy.Minimize(x * y)
         constr = [(y * cvxpy.one_minus_pos(x / y)) ** 2 >= 1, x >= y/3]
         problem = cvxpy.Problem(obj, constr)
         # smoke test.
-        problem.solve(gp=True)
+        problem.solve(SOLVER, gp=True)
 
-    def test_paper_example_eye_minus_inv(self):
+    def test_paper_example_eye_minus_inv(self) -> None:
         X = cvxpy.Variable((2, 2), pos=True)
         obj = cvxpy.Minimize(cvxpy.trace(cvxpy.eye_minus_inv(X)))
-        constr = [cvxpy.geo_mean(cvxpy.diag(X)) == 0.1]
+        constr = [cvxpy.geo_mean(cvxpy.diag(X)) == 0.1,
+                  cvxpy.geo_mean(cvxpy.hstack([X[0, 1], X[1, 0]])) == 0.1]
         problem = cvxpy.Problem(obj, constr)
-        # smoke test.
-        problem.solve(gp=True, solver="SCS")
+        problem.solve(gp=True, solver="ECOS")
+        np.testing.assert_almost_equal(X.value, 0.1*np.ones((2, 2)), decimal=3)
+        self.assertAlmostEqual(problem.value, 2.25)
 
-    def test_paper_example_exp_log(self):
+    def test_simpler_eye_minus_inv(self) -> None:
+        X = cvxpy.Variable((2, 2), pos=True)
+        obj = cvxpy.Minimize(cvxpy.trace(cvxpy.eye_minus_inv(X)))
+        constr = [cvxpy.diag(X) == 0.1,
+                  cvxpy.hstack([X[0, 1], X[1, 0]]) == 0.1]
+        problem = cvxpy.Problem(obj, constr)
+        problem.solve(gp=True, solver="ECOS")
+        np.testing.assert_almost_equal(X.value, 0.1*np.ones((2, 2)), decimal=3)
+        self.assertAlmostEqual(problem.value, 2.25)
+
+    def test_paper_example_exp_log(self) -> None:
         x = cvxpy.Variable(pos=True)
         y = cvxpy.Variable(pos=True)
         obj = cvxpy.Minimize(x * y)
         constr = [cvxpy.exp(y/x) <= cvxpy.log(y)]
         problem = cvxpy.Problem(obj, constr)
         # smoke test.
-        problem.solve(gp=True)
+        problem.solve(SOLVER, gp=True)
 
-    def test_pf_matrix_completion(self):
+    def test_pf_matrix_completion(self) -> None:
         X = cvxpy.Variable((3, 3), pos=True)
         obj = cvxpy.Minimize(cvxpy.pf_eigenvalue(X))
         known_indices = tuple(zip(*[[0, 0], [0, 2], [1, 1], [2, 0], [2, 1]]))
@@ -453,9 +470,9 @@ class TestDgp2Dcp(BaseTest):
         ]
         problem = cvxpy.Problem(obj, constr)
         # smoke test.
-        problem.solve(gp=True)
+        problem.solve(SOLVER, gp=True)
 
-    def test_rank_one_nmf(self):
+    def test_rank_one_nmf(self) -> None:
         X = cvxpy.Variable((3, 3), pos=True)
         x = cvxpy.Variable((3,), pos=True)
         y = cvxpy.Variable((3,), pos=True)
@@ -474,9 +491,9 @@ class TestDgp2Dcp(BaseTest):
         ]
         # smoke test.
         prob = cvxpy.Problem(cvxpy.Minimize(objective), constraints)
-        prob.solve(gp=True)
+        prob.solve(SOLVER, gp=True)
 
-    def test_documentation_prob(self):
+    def test_documentation_prob(self) -> None:
         x = cvxpy.Variable(pos=True)
         y = cvxpy.Variable(pos=True)
         z = cvxpy.Variable(pos=True)
@@ -486,9 +503,9 @@ class TestDgp2Dcp(BaseTest):
           4 * x * y * z + 2 * x * z <= 10, x <= 2*y, y <= 2*x, z >= 1]
         problem = cvxpy.Problem(cvxpy.Maximize(objective_fn), constraints)
         # Smoke test.
-        problem.solve(gp=True)
+        problem.solve(SOLVER, gp=True)
 
-    def test_solver_error(self):
+    def test_solver_error(self) -> None:
         x = cvxpy.Variable(pos=True)
         y = cvxpy.Variable(pos=True)
         prod = x * y
@@ -499,44 +516,109 @@ class TestDgp2Dcp(BaseTest):
         dgp_soln = dgp2dcp.invert(soln, inverse_data)
         self.assertEqual(dgp_soln.status, SOLVER_ERROR)
 
-    def test_sum_scalar(self):
+    def test_sum_scalar(self) -> None:
         w = cvxpy.Variable(pos=True)
         h = cvxpy.Variable(pos=True)
         problem = cvxpy.Problem(cvxpy.Minimize(h),
                                 [w*h >= 10, cvxpy.sum(w) <= 5])
-        problem.solve(gp=True)
+        problem.solve(SOLVER, gp=True)
         np.testing.assert_almost_equal(problem.value, 2)
         np.testing.assert_almost_equal(h.value, 2)
         np.testing.assert_almost_equal(w.value, 5)
 
-    def test_sum_vector(self):
+    def test_sum_vector(self) -> None:
         w = cvxpy.Variable(2, pos=True)
         h = cvxpy.Variable(2, pos=True)
         problem = cvxpy.Problem(cvxpy.Minimize(cvxpy.sum(h)),
                                 [cvxpy.multiply(w, h) >= 10,
                                 cvxpy.sum(w) <= 10])
-        problem.solve(gp=True)
+        problem.solve(SOLVER, gp=True)
         np.testing.assert_almost_equal(problem.value, 4)
         np.testing.assert_almost_equal(h.value, np.array([2, 2]))
         np.testing.assert_almost_equal(w.value, np.array([5, 5]))
 
-    def test_sum_matrix(self):
+    def test_sum_squares_vector(self) -> None:
+        w = cvxpy.Variable(2, pos=True)
+        h = cvxpy.Variable(2, pos=True)
+        problem = cvxpy.Problem(cvxpy.Minimize(cvxpy.sum_squares(h)),
+                                [cvxpy.multiply(w, h) >= 10,
+                                cvxpy.sum(w) <= 10])
+        problem.solve(SOLVER, gp=True)
+        np.testing.assert_almost_equal(problem.value, 8)
+        np.testing.assert_almost_equal(h.value, np.array([2, 2]))
+        np.testing.assert_almost_equal(w.value, np.array([5, 5]))
+
+    def test_sum_matrix(self) -> None:
         w = cvxpy.Variable((2, 2), pos=True)
         h = cvxpy.Variable((2, 2), pos=True)
         problem = cvxpy.Problem(cvxpy.Minimize(cvxpy.sum(h)),
                                 [cvxpy.multiply(w, h) >= 10,
                                 cvxpy.sum(w) <= 20])
-        problem.solve(gp=True)
+        problem.solve(SOLVER, gp=True)
         np.testing.assert_almost_equal(problem.value, 8)
         np.testing.assert_almost_equal(h.value, np.array([[2, 2], [2, 2]]))
         np.testing.assert_almost_equal(w.value, np.array([[5, 5], [5, 5]]))
 
-    def test_trace(self):
+    def test_trace(self) -> None:
         w = cvxpy.Variable((1, 1), pos=True)
         h = cvxpy.Variable(pos=True)
         problem = cvxpy.Problem(cvxpy.Minimize(h),
                                 [w*h >= 10, cvxpy.trace(w) <= 5])
-        problem.solve(gp=True)
+        problem.solve(SOLVER, gp=True)
         np.testing.assert_almost_equal(problem.value, 2)
         np.testing.assert_almost_equal(h.value, 2)
         np.testing.assert_almost_equal(w.value, np.array([[5]]))
+
+    def test_parameter(self) -> None:
+        param = cvxpy.Parameter(pos=True)
+        param.value = 1.0
+        dgp = cvxpy.Problem(cvxpy.Minimize(param), [])
+        dgp2dcp = cvxpy.reductions.Dgp2Dcp(dgp)
+        dcp = dgp2dcp.reduce()
+        self.assertAlmostEqual(dcp.parameters()[0].value, np.log(param.value))
+
+        x = cvxpy.Variable(pos=True)
+        problem = cvxpy.Problem(cvxpy.Minimize(x), [x == param])
+        problem.solve(SOLVER, gp=True)
+        self.assertAlmostEqual(problem.value, 1.0)
+
+        param.value = 2.0
+        problem.solve(SOLVER, gp=True)
+        self.assertAlmostEqual(problem.value, 2.0)
+
+    def test_parameter_name(self) -> None:
+        param = cvxpy.Parameter(pos=True, name='alpha')
+        param.value = 1.0
+        dgp = cvxpy.Problem(cvxpy.Minimize(param), [])
+        dgp2dcp = cvxpy.reductions.Dgp2Dcp(dgp)
+        dcp = dgp2dcp.reduce()
+        self.assertAlmostEqual(dcp.parameters()[0].name(), 'alpha')
+
+    def test_gmatmul(self) -> None:
+        x = cvxpy.Variable(2, pos=True)
+        A = np.array([[-5., 2.], [1., -3.]])
+        b = np.array([3, 2])
+        expr = cvxpy.gmatmul(A, x)
+        x.value = b
+        self.assertItemsAlmostEqual(expr.value, [3**-5*2**2, 3./8])
+        A_par = cvxpy.Parameter((2, 2), value=A)
+        self.assertItemsAlmostEqual(cvxpy.gmatmul(A_par, x).value,
+                                    [3**-5*2**2, 3./8])
+        x.value = None
+
+        prob = cvxpy.Problem(cvxpy.Minimize(1.0), [expr == b])
+        prob.solve(solver=SOLVER, gp=True)
+        sltn = np.exp(np.linalg.solve(A, np.log(b)))
+        self.assertItemsAlmostEqual(x.value, sltn)
+
+    def test_xexp(self) -> None:
+        x = cvxpy.Variable(2, pos=True)
+        b = np.array([1, 0.5])
+        expr = cvxpy.xexp(x)
+        x.value = b
+        self.assertItemsAlmostEqual(expr.value, [np.e, 0.5 * np.e**.5])
+
+        prob = cvxpy.Problem(cvxpy.Minimize(cvxpy.prod(expr)),
+                             [x >= b])
+        prob.solve(solver=SOLVER, gp=True)
+        self.assertItemsAlmostEqual(expr.value, [np.e, 0.5 * np.e**.5])
