@@ -14,26 +14,26 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
-import cvxpy as cp
-from cvxpy.atoms.affine.add_expr import AddExpression
-from cvxpy.expressions.variable import Variable
-from cvxpy.expressions.constants import Constant
-from cvxpy.expressions.constants import Parameter
-from cvxpy import Problem, Minimize
-import cvxpy.interface.matrix_utilities as intf
-import cvxpy.settings as s
-from cvxpy.tests.base_test import BaseTest
+import warnings
+
 import numpy as np
 import scipy.sparse as sp
-import warnings
-import sys
-PY35 = sys.version_info >= (3, 5)
+
+import cvxpy as cp
+import cvxpy.interface.matrix_utilities as intf
+import cvxpy.settings as s
+from cvxpy import Minimize, Problem
+from cvxpy.atoms.affine.add_expr import AddExpression
+from cvxpy.expressions.constants import Constant, Parameter
+from cvxpy.expressions.variable import Variable
+from cvxpy.tests.base_test import BaseTest
+from cvxpy.utilities.eigvals import gershgorin_psd_check
 
 
 class TestExpressions(BaseTest):
     """ Unit tests for the expression/expression module. """
 
-    def setUp(self):
+    def setUp(self) -> None:
         self.a = Variable(name='a')
 
         self.x = Variable(2, name='x')
@@ -46,7 +46,7 @@ class TestExpressions(BaseTest):
         self.intf = intf.DEFAULT_INTF
 
     # Test the Variable class.
-    def test_variable(self):
+    def test_variable(self) -> None:
         x = Variable(2)
         y = Variable(2)
         assert y.name() != x.name()
@@ -100,7 +100,7 @@ class TestExpressions(BaseTest):
         self.assertEqual(str(cm.exception),
                          "Variable name 1 must be a string.")
 
-    def test_assign_var_value(self):
+    def test_assign_var_value(self) -> None:
         """Test assigning a value to a variable.
         """
         # Scalar variable.
@@ -132,7 +132,7 @@ class TestExpressions(BaseTest):
         self.assertEqual(str(cm.exception), "Variable value must be nonnegative.")
 
     # Test tranposing variables.
-    def test_transpose_variable(self):
+    def test_transpose_variable(self) -> None:
         var = self.a.T
         self.assertEqual(var.name(), "a")
         self.assertEqual(var.shape, tuple())
@@ -171,7 +171,7 @@ class TestExpressions(BaseTest):
         self.assertEqual(var.shape, (2, 1))
 
     # Test the Constant class.
-    def test_constants(self):
+    def test_constants(self) -> None:
         c = Constant(2.0)
         self.assertEqual(c.name(), str(2.0))
 
@@ -205,28 +205,88 @@ class TestExpressions(BaseTest):
         c = Constant([1, 2])
         self.assertEqual(c.shape, (2,))
         A = Constant([[1, 1], [1, 1]])
-        exp = c.T*A*c
+        exp = c.T @ A @ c
         self.assertEqual(exp.sign, s.NONNEG)
-        self.assertEqual((c.T*c).sign, s.NONNEG)
+        self.assertEqual((c.T @ c).sign, s.NONNEG)
         exp = c.T.T
         self.assertEqual(exp.sign, s.NONNEG)
-        exp = c.T*self.A
+        exp = c.T @ self.A
         self.assertEqual(exp.sign, s.UNKNOWN)
 
         # Test repr.
         self.assertEqual(repr(c), "Constant(CONSTANT, NONNEGATIVE, (2,))")
 
-    def test_1D_array(self):
+    def test_constant_psd_nsd(self):
+        n = 5
+        np.random.randn(0)
+        U = np.random.randn(n, n)
+        U = U @ U.T
+        (evals, U) = np.linalg.eigh(U)  # U is now an orthogonal matrix
+
+        # Try four indefinite matrices with different eigenvalue
+        # spread around the origin.
+        v1 = np.array([3, 2, 1, 1e-8, -1])
+        P = Constant(U @ np.diag(v1) @ U.T)
+        self.assertFalse(P.is_psd())
+        self.assertFalse(P.is_nsd())
+        v2 = np.array([3, 2, 2, 1e-6, -1])
+        P = Constant(U @ np.diag(v2) @ U.T)
+        self.assertFalse(P.is_psd())
+        self.assertFalse(P.is_nsd())
+        v3 = np.array([3, 2, 2, 1e-4, -1e-6])
+        P = Constant(U @ np.diag(v3) @ U.T)
+        self.assertFalse(P.is_psd())
+        self.assertFalse(P.is_nsd())
+        v4 = np.array([-1, 3, 0, 0, 0])
+        P = Constant(U @ np.diag(v4) @ U.T)
+        self.assertFalse(P.is_psd())
+        self.assertFalse(P.is_nsd())
+
+        # Try a test case given in GitHub issue 1451.
+        # (Should be equivalent to v4 above).
+        P = Constant(np.array([[1, 2], [2, 1]]))
+        x = Variable(shape=(2,))
+        expr = cp.quad_form(x, P)
+        self.assertFalse(expr.is_dcp())
+        self.assertFalse((-expr).is_dcp())
+        self.assertFalse(gershgorin_psd_check(P.value, tol=0.99))
+
+        # Useful Gershgorin disc check
+        P = Constant(np.array([[2, 1], [1, 2]]))
+        self.assertTrue(gershgorin_psd_check(P.value, tol=0.0))
+
+        # Verify good behavior for large eigenvalues
+        P = Constant(np.diag(9*[1e-4] + [-1e4]))
+        self.assertFalse(P.is_psd())
+        self.assertFalse(P.is_nsd())
+
+        # Check a case when the matrix is in fact PSD.
+        P = Constant(np.ones(shape=(5, 5)))
+        self.assertTrue(P.is_psd())
+        self.assertFalse(P.is_nsd())
+
+        # Check with sparse inputs
+        P = Constant(sp.eye(10))
+        self.assertTrue(gershgorin_psd_check(P.value, s.EIGVAL_TOL))
+        self.assertTrue(P.is_psd())
+        self.assertTrue((-P).is_nsd())
+        Q = -s.EIGVAL_TOL/2 * P
+        self.assertTrue(gershgorin_psd_check(Q.value, s.EIGVAL_TOL))
+        Q = -1.1*s.EIGVAL_TOL*P
+        self.assertFalse(gershgorin_psd_check(Q.value, s.EIGVAL_TOL))
+        self.assertFalse(Q.is_psd())
+
+    def test_1D_array(self) -> None:
         """Test NumPy 1D arrays as constants.
         """
         c = np.array([1, 2])
         p = Parameter(2)
         p.value = [1, 1]
-        self.assertEqual((c*p).value, 3)
-        self.assertEqual((c*self.x).shape, tuple())
+        self.assertEqual((c @ p).value, 3)
+        self.assertEqual((c @ self.x).shape, tuple())
 
     # Test Parameter class on good inputs.
-    def test_parameters_successes(self):
+    def test_parameters_successes(self) -> None:
         # Parameter names and dimensions
         p = Parameter(name='p')
         self.assertEqual(p.name(), "p")
@@ -254,7 +314,7 @@ class TestExpressions(BaseTest):
         p.value = sp.csc_matrix(np.eye(2))
         self.assertItemsAlmostEqual(p.value.todense(), np.eye(2), places=10)
 
-    def test_psd_nsd_parameters(self):
+    def test_psd_nsd_parameters(self) -> None:
         # Test valid rank-deficeint PSD parameter.
         np.random.seed(42)
         a = np.random.normal(size=(100, 95))
@@ -274,16 +334,6 @@ class TestExpressions(BaseTest):
         p.value = A
         self.assertItemsAlmostEqual(p.value, A)
 
-        # Test invalid PSD parameter
-        with self.assertRaises(Exception) as cm:
-            p = Parameter((2, 2), PSD=True, value=[[1, 0], [0, -1]])
-        self.assertEqual(str(cm.exception), "Parameter value must be positive semidefinite.")
-
-        # Test invalid NSD parameter
-        with self.assertRaises(Exception) as cm:
-            p = Parameter((2, 2), NSD=True, value=[[1, 0], [0, -1]])
-        self.assertEqual(str(cm.exception), "Parameter value must be negative semidefinite.")
-
         # Test arithmetic.
         p = Parameter(shape=(2, 2), PSD=True)
         self.assertTrue((2*p).is_psd())
@@ -291,8 +341,29 @@ class TestExpressions(BaseTest):
         self.assertTrue((-p).is_nsd())
         self.assertTrue(((-2)*(-p)).is_psd())
 
+        # Test invalid PSD and NSD parameters
+        n = 5
+        P = Parameter(shape=(n, n), PSD=True)
+        N = Parameter(shape=(n, n), NSD=True)
+        np.random.randn(0)
+        U = np.random.randn(n, n)
+        U = U @ U.T
+        (evals, U) = np.linalg.eigh(U)  # U is now an orthogonal matrix
+        v1 = np.array([3, 2, 1, 1e-8, -1])
+        v2 = np.array([3, 2, 2, 1e-6, -1])
+        v3 = np.array([3, 2, 2, 1e-4, -1e-6])
+        v4 = np.array([-1, 3, 0, 0, 0])
+        vs = [v1, v2, v3, v4]
+        for vi in vs:
+            with self.assertRaises(Exception) as cm:
+                P.value = U @ np.diag(vi) @ U.T
+            self.assertEqual(str(cm.exception), "Parameter value must be positive semidefinite.")
+            with self.assertRaises(Exception) as cm:
+                N.value = -U @ np.diag(vi) @ U.T
+            self.assertEqual(str(cm.exception), "Parameter value must be negative semidefinite.")
+
     # Test the Parameter class on bad inputs.
-    def test_parameters_failures(self):
+    def test_parameters_failures(self) -> None:
         p = Parameter(name='p')
         self.assertEqual(p.name(), "p")
         self.assertEqual(p.shape, tuple())
@@ -348,7 +419,7 @@ class TestExpressions(BaseTest):
             p = Parameter((2, 2), symmetric=True, value=[[1, 1], [-1, -1]])
         self.assertEqual(str(cm.exception), "Parameter value must be symmetric.")
 
-    def test_symmetric(self):
+    def test_symmetric(self) -> None:
         """Test symmetric variables.
         """
         with self.assertRaises(Exception) as cm:
@@ -382,7 +453,7 @@ class TestExpressions(BaseTest):
         expr = cp.promote(Variable(), (2, 2))
         assert expr.is_symmetric()
 
-    def test_hermitian(self):
+    def test_hermitian(self) -> None:
         """Test Hermitian variables.
         """
         with self.assertRaises(Exception) as cm:
@@ -414,7 +485,7 @@ class TestExpressions(BaseTest):
         expr = cp.promote(Variable(), (2, 2))
         assert expr.is_hermitian()
 
-    def test_round_attr(self):
+    def test_round_attr(self) -> None:
         """Test rounding for attributes.
         """
         # Nonpos
@@ -475,7 +546,7 @@ class TestExpressions(BaseTest):
         self.assertEqual(A.is_nsd(), True)
 
     # Test the AddExpresion class.
-    def test_add_expression(self):
+    def test_add_expression(self) -> None:
         # Vectors
         c = Constant([2, 2])
         exp = self.x + c
@@ -514,7 +585,7 @@ class TestExpressions(BaseTest):
         self.assertEqual(repr(exp), "Expression(AFFINE, UNKNOWN, (2,))")
 
     # Test the SubExpresion class.
-    def test_sub_expression(self):
+    def test_sub_expression(self) -> None:
         # Vectors
         c = Constant([2, 2])
         exp = self.x - c
@@ -545,12 +616,12 @@ class TestExpressions(BaseTest):
         self.assertEqual(repr(self.x - c), "Expression(AFFINE, UNKNOWN, (2,))")
 
     # Test the MulExpresion class.
-    def test_mul_expression(self):
+    def test_mul_expression(self) -> None:
         # Vectors
         c = Constant([[2], [2]])
-        exp = c*self.x
+        exp = c @ self.x
         self.assertEqual(exp.curvature, s.AFFINE)
-        self.assertEqual((c[0]*self.x).sign, s.UNKNOWN)
+        self.assertEqual((c[0] @ self.x).sign, s.UNKNOWN)
         # self.assertEqual(exp.canonical_form[0].shape, (1, 1))
         # self.assertEqual(exp.canonical_form[1], [])
         # self.assertEqual(exp.name(), c.name() + " * " + self.x.name())
@@ -558,30 +629,55 @@ class TestExpressions(BaseTest):
 
         # Incompatible dimensions
         with self.assertRaises(ValueError):
-            ([2, 2, 3]*self.x)
+            ([2, 2, 3] @ self.x)
 
         # Matrices: Incompatible dimensions
         with self.assertRaises(ValueError):
-            Constant([[2, 1], [2, 2]]) * self.C
+            Constant([[2, 1], [2, 2]]) @ self.C
 
         # Affine times affine is okay
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
-            q = self.A * self.B
+            q = self.A @ self.B
             self.assertTrue(q.is_quadratic())
 
         # Constant expressions
         T = Constant([[1, 2, 3], [3, 5, 5]])
-        exp = (T + T) * self.B
+        exp = (T + T) @ self.B
         self.assertEqual(exp.curvature, s.AFFINE)
         self.assertEqual(exp.shape, (3, 2))
 
         # Expression that would break sign multiplication without promotion.
         c = Constant([[2], [2], [-2]])
-        exp = [[1], [2]] + c*self.C
+        exp = [[1], [2]] + c @ self.C
         self.assertEqual(exp.sign, s.UNKNOWN)
 
-    def test_matmul_expression(self):
+        # By default, warnings are raised if we access matmul from *.
+        c = Constant([[2], [2]])
+        with warnings.catch_warnings(record=True) as w:
+            c * self.x
+            self.assertEqual(2, len(w))
+            self.assertEqual(w[0].category, UserWarning)
+            self.assertEqual(w[1].category, DeprecationWarning)
+            # repeat, to make sure warnings continue to be displayed
+            c * self.x
+            self.assertEqual(4, len(w))
+            self.assertEqual(w[2].category, UserWarning)
+            self.assertEqual(w[3].category, DeprecationWarning)
+            # suppress one of the two warnings
+            warnings.simplefilter('ignore', DeprecationWarning)
+            c * self.x
+            self.assertEqual(5, len(w))
+            # suppress both warnings
+            warnings.simplefilter('ignore', UserWarning)
+            c * self.x
+            self.assertEqual(len(w), 5)
+            # verify that an error can be raised.
+            warnings.simplefilter("error", UserWarning)
+            with self.assertRaises(UserWarning):
+                c * self.x
+
+    def test_matmul_expression(self) -> None:
         """Test matmul function, corresponding to .__matmul__( operator.
         """
         # Vectors
@@ -651,7 +747,7 @@ class TestExpressions(BaseTest):
         assert v.shape == col_scalar.shape == col_scalar.T.shape
 
     # Test the DivExpresion class.
-    def test_div_expression(self):
+    def test_div_expression(self) -> None:
         # Vectors
         exp = self.x/2
         self.assertEqual(exp.curvature, s.AFFINE)
@@ -663,9 +759,9 @@ class TestExpressions(BaseTest):
 
         with self.assertRaises(Exception) as cm:
             (self.x/[2, 2, 3])
-        print((cm.exception))
+        print(cm.exception)
         self.assertRegex(str(cm.exception),
-                                 "Incompatible shapes for division.*")
+                         "Incompatible shapes for division.*")
 
         c = Constant([3.0, 4.0, 12.0])
         self.assertItemsAlmostEqual(
@@ -693,8 +789,23 @@ class TestExpressions(BaseTest):
         self.assertEqual((Constant(2)*rho).sign, s.NONNEG)
         self.assertEqual((rho/2).sign, s.NONNEG)
 
+        # Broadcasting.
+        x = cp.Variable((3, 3))
+        c = np.arange(1, 4)[:, None]
+        expr = x / c
+        self.assertEqual((3, 3), expr.shape)
+        x.value = np.ones((3, 3))
+        A = np.ones((3, 3)) / c
+        self.assertItemsAlmostEqual(A, expr.value)
+
+        with self.assertRaises(Exception) as cm:
+            (x/c[:, 0])
+        print(cm.exception)
+        self.assertRegex(str(cm.exception),
+                         "Incompatible shapes for division.*")
+
     # Test the NegExpression class.
-    def test_neg_expression(self):
+    def test_neg_expression(self) -> None:
         # Vectors
         exp = -self.x
         self.assertEqual(exp.curvature, s.AFFINE)
@@ -713,7 +824,7 @@ class TestExpressions(BaseTest):
         self.assertEqual(exp.shape, (3, 2))
 
     # Test promotion of scalar constants.
-    def test_scalar_const_promotion(self):
+    def test_scalar_const_promotion(self) -> None:
         # Vectors
         exp = self.x + 2
         self.assertEqual(exp.curvature, s.AFFINE)
@@ -739,7 +850,7 @@ class TestExpressions(BaseTest):
         self.assertEqual(exp.shape, (2, 2))
 
     # Test indexing expression.
-    def test_index_expression(self):
+    def test_index_expression(self) -> None:
         # Tuple of integers as key.
         exp = self.x[1]
         # self.assertEqual(exp.name(), "x[1,0]")
@@ -828,18 +939,18 @@ class TestExpressions(BaseTest):
         self.assertEqual(exp.shape, tuple())
 
         c = Constant([[1, 2], [3, 4]])
-        exp = (c*self.x)[1]
+        exp = (c @ self.x)[1]
         # self.assertEqual(exp.name(), "[[2], [4]] * x[0:,0]")
         self.assertEqual(exp.curvature, s.AFFINE)
         self.assertEqual(exp.shape, tuple())
 
         c = Constant([[1, 2], [3, 4]])
-        exp = (c*self.a)[1, 0:1]
+        exp = (c * self.a)[1, 0:1]
         # self.assertEqual(exp.name(), "2 * a")
         self.assertEqual(exp.curvature, s.AFFINE)
         self.assertEqual(exp.shape, (1,))
 
-    def test_none_idx(self):
+    def test_none_idx(self) -> None:
         """Test None as index.
         """
         expr = self.a[None, None]
@@ -855,7 +966,7 @@ class TestExpressions(BaseTest):
         self.assertEqual(expr.shape, (1, 2))
         self.assertItemsAlmostEqual(expr.value, [1, 2])
 
-    def test_out_of_bounds(self):
+    def test_out_of_bounds(self) -> None:
         """Test out of bounds indices.
         """
         with self.assertRaises(Exception) as cm:
@@ -879,7 +990,7 @@ class TestExpressions(BaseTest):
         exp = self.C[:, -199:-3]
         self.assertEqual(exp.shape, (3, 0))
 
-    def test_neg_indices(self):
+    def test_neg_indices(self) -> None:
         """Test negative indices.
         """
         c = Constant([[1, 2], [3, 4]])
@@ -902,7 +1013,7 @@ class TestExpressions(BaseTest):
 
         x = Variable(4)
         self.assertEqual(x[::-1].shape, (4,))
-        Problem(Minimize(0), [x[::-1] == c]).solve()
+        Problem(Minimize(0), [x[::-1] == c]).solve(solver=cp.SCS)
         self.assertItemsAlmostEqual(x.value, [4, 3, 2, 1])
 
         x = Variable(2)
@@ -920,7 +1031,7 @@ class TestExpressions(BaseTest):
         self.assertEqual(expr.shape, (2,))
         self.assertItemsAlmostEqual(expr.value, [3, 1])
 
-    def test_logical_indices(self):
+    def test_logical_indices(self) -> None:
         """Test indexing with boolean arrays.
         """
         A = np.array([[1, 2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12]])
@@ -972,7 +1083,7 @@ class TestExpressions(BaseTest):
         self.assertItemsAlmostEqual(A[np.array([True, True, True]),
                                       np.array([True, False, True, True])], expr.value)
 
-    def test_selector_list_indices(self):
+    def test_selector_list_indices(self) -> None:
         """Test indexing with lists/ndarrays of indices.
         """
         A = np.array([[1, 2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12]])
@@ -1026,7 +1137,7 @@ class TestExpressions(BaseTest):
         self.assertEqual(expr.sign, s.NONNEG)
         self.assertItemsAlmostEqual(A[np.array([0, 1]), np.array([1, 3])], expr.value)
 
-    def test_powers(self):
+    def test_powers(self) -> None:
         exp = self.x**2
         self.assertEqual(exp.curvature, s.CONVEX)
         exp = self.x**0.5
@@ -1034,7 +1145,7 @@ class TestExpressions(BaseTest):
         exp = self.x**-1
         self.assertEqual(exp.curvature, s.CONVEX)
 
-    def test_sum(self):
+    def test_sum(self) -> None:
         """Test cvxpy sum function.
         """
         self.a.value = 1
@@ -1045,7 +1156,7 @@ class TestExpressions(BaseTest):
         expr = cp.sum(self.x)
         self.assertEqual(expr.value, 3)
 
-    def test_var_copy(self):
+    def test_var_copy(self) -> None:
         """Test the copy function for variable types.
         """
         x = Variable((3, 4), name="x")
@@ -1057,7 +1168,7 @@ class TestExpressions(BaseTest):
         y = x.copy()
         self.assertEqual(y.shape, (5, 5))
 
-    def test_param_copy(self):
+    def test_param_copy(self) -> None:
         """Test the copy function for Parameters.
         """
         x = Parameter((3, 4), name="x", nonneg=True)
@@ -1066,7 +1177,7 @@ class TestExpressions(BaseTest):
         self.assertEqual(y.name(), "x")
         self.assertEqual(y.sign, "NONNEGATIVE")
 
-    def test_constant_copy(self):
+    def test_constant_copy(self) -> None:
         """Test the copy function for Constants.
         """
         x = Constant(2)
@@ -1074,13 +1185,13 @@ class TestExpressions(BaseTest):
         self.assertEqual(y.shape, tuple())
         self.assertEqual(y.value, 2)
 
-    def test_is_pwl(self):
+    def test_is_pwl(self) -> None:
         """Test is_pwl()
         """
         A = np.ones((2, 3))
         b = np.ones(2)
 
-        expr = A * self.y - b
+        expr = A @ self.y - b
         self.assertEqual(expr.is_pwl(), True)
 
         expr = cp.maximum(1, 3 * self.y)
@@ -1094,3 +1205,80 @@ class TestExpressions(BaseTest):
 
         expr = cp.pnorm(3 * self.y ** 2, 1)
         self.assertEqual(expr.is_pwl(), False)
+
+    def test_broadcast_mul(self) -> None:
+        """Test multiply broadcasting.
+        """
+        y = Parameter((3, 1))
+        z = Variable((1, 3))
+        y.value = np.arange(3)[:, None]
+        z.value = (np.arange(3) - 1)[None, :]
+        expr = cp.multiply(y, z)
+        self.assertItemsAlmostEqual(expr.value, y.value * z.value)
+
+        prob = cp.Problem(cp.Minimize(cp.sum(expr)), [z == z.value])
+        prob.solve(solver=cp.SCS)
+        self.assertItemsAlmostEqual(expr.value, y.value * z.value)
+
+        np.random.seed(0)
+        m, n = 3, 4
+        A = np.random.rand(m, n)
+
+        col_scale = Variable(n)
+
+        with self.assertRaises(ValueError) as cm:
+            cp.multiply(A, col_scale)
+        self.assertEqual(str(cm.exception), "Cannot broadcast dimensions  (3, 4) (4,)")
+
+        col_scale = Variable([1, n])
+        C = cp.multiply(A, col_scale)
+        self.assertEqual(C.shape, (m, n))
+
+        row_scale = Variable([m, 1])
+        R = cp.multiply(A, row_scale)
+        self.assertEqual(R.shape, (m, n))
+
+    def test_broadcast_add(self) -> None:
+        """Test addition broadcasting.
+        """
+        y = Parameter((3, 1))
+        z = Variable((1, 3))
+        y.value = np.arange(3)[:, None]
+        z.value = (np.arange(3) - 1)[None, :]
+        expr = y + z
+        self.assertItemsAlmostEqual(expr.value, y.value + z.value)
+
+        prob = cp.Problem(cp.Minimize(cp.sum(expr)), [z == z.value])
+        prob.solve(solver=cp.SCS)
+        self.assertItemsAlmostEqual(expr.value, y.value + z.value)
+
+        np.random.seed(0)
+        m, n = 3, 4
+        A = np.random.rand(m, n)
+
+        col_scale = Variable(n)
+
+        with self.assertRaises(ValueError) as cm:
+            A + col_scale
+        self.assertEqual(str(cm.exception), "Cannot broadcast dimensions  (3, 4) (4,)")
+
+        col_scale = Variable([1, n])
+        C = A + col_scale
+        self.assertEqual(C.shape, (m, n))
+
+        row_scale = Variable([m, 1])
+        R = A + row_scale
+        self.assertEqual(R.shape, (m, n))
+
+    def test_log_log_curvature(self) -> None:
+        """Test that the curvature string is populated for log-log expressions.
+        """
+        x = Variable(pos=True)
+        monomial = x*x*x
+        assert monomial.curvature == s.LOG_LOG_AFFINE
+
+        posynomial = x*x*x + x
+        assert posynomial.curvature == s.LOG_LOG_CONVEX
+
+        llcv = 1/(x*x*x + x)
+        assert llcv.curvature == s.LOG_LOG_CONCAVE
