@@ -14,15 +14,16 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+import numpy as np
+
 from cvxpy.expressions.constants import Constant
 from cvxpy.expressions.variable import Variable
 from cvxpy.utilities.power_tools import gm_constrs
-import numpy as np
 
 
 def power_canon(expr, args):
     x = args[0]
-    p = expr.p
+    p = expr.p_rational
     w = expr.w
 
     if p == 1:
