@@ -17,8 +17,10 @@ limitations under the License.
 from cvxpy.atoms.elementwise.abs import abs
 from cvxpy.atoms.elementwise.power import power
 from cvxpy.expressions.variable import Variable
-from cvxpy.reductions.eliminate_pwl.atom_canonicalizers.abs_canon import abs_canon
-from cvxpy.reductions.qp2quad_form.atom_canonicalizers.power_canon import power_canon
+from cvxpy.reductions.eliminate_pwl.atom_canonicalizers.abs_canon import (
+    abs_canon,)
+from cvxpy.reductions.qp2quad_form.atom_canonicalizers.power_canon import (
+    power_canon,)
 
 
 def huber_canon(expr, args):
