@@ -14,14 +14,16 @@ See the License for the specific language governing permissions and
 limitations under the License.
 """
 
+import numbers
+from collections import defaultdict
 from fractions import Fraction
+
+import numpy as np
+
 from cvxpy.atoms.affine.reshape import reshape
 from cvxpy.atoms.affine.vstack import vstack
 from cvxpy.constraints.second_order import SOC
 from cvxpy.expressions.variable import Variable
-import numpy as np
-from collections import defaultdict
-import numbers
 
 
 def gm(t, x, y):
@@ -64,8 +66,9 @@ def gm_constrs(t, x_list, p):
     d = defaultdict(lambda: Variable(t.shape))
     d[w] = t
 
-    if len(x_list) < len(w):
-        x_list += [t]
+    long_w = len(w) - len(x_list)
+    if long_w > 0:
+        x_list += [t]*long_w
 
     assert len(x_list) == len(w)
 
@@ -76,14 +79,14 @@ def gm_constrs(t, x_list, p):
             d[tuple(tmp)] = v
 
     constraints = []
-    for elem, children in list(tree.items()):
+    for elem, children in tree.items():
         if 1 not in elem:
             constraints += [gm(d[elem], d[children[0]], d[children[1]])]
 
     return constraints
 
 
-def pow_high(p, max_denom=1024):
+def pow_high(p, max_denom: int = 1024):
     """ Return (t,1,x) power tuple
 
         x <= t^(1/p) 1^(1-1/p)
@@ -97,7 +100,7 @@ def pow_high(p, max_denom=1024):
     return 1/p, (p, 1-p)
 
 
-def pow_mid(p, max_denom=1024):
+def pow_mid(p, max_denom: int = 1024):
     """ Return (x,1,t) power tuple
 
         t <= x^p 1^(1-p)
@@ -109,7 +112,7 @@ def pow_mid(p, max_denom=1024):
     return p, (p, 1-p)
 
 
-def pow_neg(p, max_denom=1024):
+def pow_neg(p, max_denom: int = 1024):
     """ Return (x,t,1) power tuple
 
         1 <= x^(p/(p-1)) t^(-1/(p-1))
@@ -122,7 +125,7 @@ def pow_neg(p, max_denom=1024):
     return p/(p-1), (p, 1-p)
 
 
-def is_power2(num):
+def is_power2(num) -> bool:
     """ Test if num is a positive integer power of 2.
 
     .. note::
@@ -148,7 +151,7 @@ def is_power2(num):
     return isinstance(num, numbers.Integral) and num > 0 and not (num & (num - 1))
 
 
-def is_dyad(frac):
+def is_dyad(frac) -> bool:
     """ Test if frac is a nonnegative dyadic fraction or integer.
 
     Examples
@@ -175,7 +178,7 @@ def is_dyad(frac):
         return False
 
 
-def is_dyad_weight(w):
+def is_dyad_weight(w) -> bool:
     """ Test if a vector is a valid dyadic weight vector.
 
         w must be nonnegative, sum to 1, and have integer or dyadic fractional elements.
@@ -192,7 +195,7 @@ def is_dyad_weight(w):
     return is_weight(w) and all(is_dyad(f) for f in w)
 
 
-def is_weight(w):
+def is_weight(w) -> bool:
     """ Test if w is a valid weight vector.
         w must have nonnegative integer or fractional elements, and sum to 1.
 
@@ -223,7 +226,15 @@ def is_weight(w):
     return valid_elems and sum(w) == 1
 
 
-def fracify(a, max_denom=1024, force_dyad=False):
+__EXCEED_DENOMINATOR_LIMIT__ = \
+    """
+    Can't reliably represent the input weight vector.
+    Try increasing `max_denom` or checking the denominators
+    of your input fractions.
+    """
+
+
+def fracify(a, max_denom: int = 1024, force_dyad: bool = False):
     """ Return a valid fractional weight tuple (and its dyadic completion)
         to represent the weights given by ``a``.
 
@@ -261,6 +272,7 @@ def fracify(a, max_denom=1024, force_dyad=False):
         That is, if w has fractions with denominators that are not a power of 2,
         and ``len(w) == n`` then w_dyad has length n+1, dyadic fractions for elements,
         and ``w_dyad[:-1]/w_dyad[n] == w``.
+        # ^ That isn't always possible, is it?
 
         Alternatively, the ratios between the
         first n elements of ``w_dyad`` are equal to the corresponding ratios between
@@ -360,17 +372,18 @@ def fracify(a, max_denom=1024, force_dyad=False):
         w_frac = tuple(Fraction(v, total) for v in a)
         d = max(v.denominator for v in w_frac)
         if d > max_denom:
-            msg = ("Can't reliably represent the input weight vector."
-                   "\nTry increasing `max_denom` or checking the denominators "
-                   "of your input fractions.")
-            raise ValueError(msg)
+            raise ValueError(__EXCEED_DENOMINATOR_LIMIT__)
     else:
         # fall through code
         w_frac = tuple(Fraction(float(v)/total).limit_denominator(max_denom) for v in a)
         if sum(w_frac) != 1:
             w_frac = make_frac(a, max_denom)
 
-    return w_frac, dyad_completion(w_frac)
+    w_dyad = dyad_completion(w_frac)
+    if max(v.denominator for v in w_dyad) > max_denom:
+        raise ValueError(__EXCEED_DENOMINATOR_LIMIT__)
+
+    return w_frac, w_dyad
 
 
 def make_frac(a, denom):
@@ -386,14 +399,13 @@ def make_frac(a, denom):
     """
 
     a = np.array(a, dtype=float)/sum(a)
-    b = [float(v*denom) for v in a]
-    b = np.array(b, dtype=int)
+    b = (denom * a).astype(int)
     err = b/float(denom) - a
 
     inds = np.argsort(err)[:(denom - sum(b))]
     b[inds] += 1
 
-    denom = np.int32(denom)
+    denom = int(denom)
     b = b.tolist()
 
     return tuple(Fraction(v, denom) for v in b)
@@ -421,16 +433,15 @@ def dyad_completion(w):
     (Fraction(1, 1), Fraction(0, 1), Fraction(0, 1), Fraction(0, 1))
     """
     w = tuple(Fraction(v) for v in w)
-    d = max(v.denominator for v in w)
-
-    # if extra_index:
-    p = next_pow2(d)
-    if p == d:
-        # the tuple of fractions is already dyadic
-        return w
-    else:
+    non_dyad_dens = [v.denominator for v in w if not is_power2(v.denominator)]
+    if len(non_dyad_dens) > 0:
         # need to add the dummy variable to represent as dyadic
-        return tuple(Fraction(v*d, p) for v in w) + (Fraction(p-d, p),)
+        d = max(non_dyad_dens)
+        p = next_pow2(d)
+        w_aug = tuple(Fraction(v*d, p) for v in w) + (Fraction(p-d, p),)
+        return dyad_completion(w_aug)
+    else:
+        return w
 
 
 def approx_error(a_orig, w_approx):
